@@ -444,10 +444,9 @@ protected:
     // std::string because I want to be as fast as possible here
     virtual void copy_until_boundary(ParsingElem pe)
     {
-        size_t pos, lines, eomsz;
+        size_t pos, lines, eomsz = 0;
         register char c;
         enum { nlsz = 1 };
-        char nl[2] = { NL, 0 };
         const char *eom = 0;
 
         enum { blksz = 4096 };
@@ -501,23 +500,28 @@ protected:
                 {
                     // not an empty row, is this a boundary?
                     block[blkpos] = 0;
-                    if(block[sl_off] == '-' &&
-                        sl_off < blkpos &&
+                    if(block[sl_off] == '-' && sl_off < blkpos &&
                          block[sl_off+1] == '-')
                     {
                         std::string Line(block+sl_off, blkpos-sl_off);
                         if(isBoundary(Line))
                         {
                             // trim last newline
-                            int i = sl_off;
-                            char a = block[--i];
-                            char b = block[--i];
-                            if(isnl(a,b))
-                                sl_off -= 2;
-                            else if(isnl(a))
+                            if (sl_off>=2) 
+                            {
+                                int i = sl_off;
+                                char a = block[--i];
+                                char b = block[--i];
+
+                                if(isnl(a,b))
+                                    sl_off -= 2;
+                                else if(isnl(a))
+                                    sl_off--;
+
+                            } else if (sl_off==1 && isnl(block[0])) {
                                 sl_off--;
-                            onBlock(block, sl_off,
-                                pe);
+                            }
+                            onBlock(block, sl_off, pe);
                             return;
                         }
                     }
@@ -675,7 +679,7 @@ private:
                     m_lastBoundary = ClosingBoundary;
                     m_bit = off + 2;
                 }
-                if(m_bit < m_eit && isnl(*m_bit)) 
+                if(m_bit < m_eit-1 && isnl(*m_bit)) 
                 {
                     char c = *m_bit++;
                     char next = *m_bit;
